import json
import logging
import shlex

from aws_gate.constants import (
    AWS_DEFAULT_PROFILE,
    AWS_DEFAULT_REGION,
    DEFAULT_OS_USER,
    DEFAULT_SSH_PORT,
    DEFAULT_KEY_ALGORITHM,
    DEFAULT_KEY_SIZE,
    PLUGIN_INSTALL_PATH,
    DEBUG,
    DEFAULT_GATE_KEY_PATH,
)
from aws_gate.decorators import (
    plugin_version,
    plugin_required,
    valid_aws_profile,
    valid_aws_region,
)
from aws_gate.query import query_instance
from aws_gate.session_common import BaseSession
from aws_gate.ssh_common import SshKey, SshKeyUploader
from aws_gate.utils import (
    get_aws_client,
    get_aws_resource,
    fetch_instance_details_from_config,
    get_instance_details,
    execute,
)

logger = logging.getLogger(__name__)


class SshSession(BaseSession):
    def __init__(
        self,
        instance_id,
        ssm=None,
        region_name=AWS_DEFAULT_REGION,
        profile_name=AWS_DEFAULT_PROFILE,
        port=DEFAULT_SSH_PORT,
        user=DEFAULT_OS_USER,
        command=None,
    ):
        self._instance_id = instance_id
        self._region_name = region_name
        self._profile_name = profile_name if profile_name is not None else ""
        self._ssm = ssm
        self._port = port
        self._user = user
        self._command = command

        self._ssh_cmd = None

        self._session_parameters = {
            "Target": self._instance_id,
            "DocumentName": "AWS-StartSSHSession",
            "Parameters": {"portNumber": [str(self._port)]},
        }

    def _build_ssh_command(self):
        cmd = [
            "ssh",
            "-l",
            self._user,
            "-p",
            str(self._port),
            "-F",
            "/dev/null",
            "-i",
            DEFAULT_GATE_KEY_PATH,
        ]

        if DEBUG:
            cmd.append("-vv")
        else:
            cmd.append("-q")

        proxy_command_args = [
            PLUGIN_INSTALL_PATH,
            json.dumps(self._response),
            self._region_name,
            "StartSession",
            self._profile_name,
            json.dumps(self._session_parameters),
            self._ssm.meta.endpoint_url,
        ]
        proxy_command = " ".join(shlex.quote(i) for i in proxy_command_args)

        ssh_options = [
            "IdentitiesOnly=yes",
            "UserKnownHostsFile=/dev/null",
            "StrictHostKeyChecking=no",
            "ProxyCommand={}".format(proxy_command),
        ]

        for ssh_option in ssh_options:
            cmd.append("-o")
            cmd.append(ssh_option)

        cmd.append(self._instance_id)

        if self._command:
            cmd.append("--")
            cmd.extend(self._command)

        return cmd

    def open(self):
        self._ssh_cmd = self._build_ssh_command()

        return execute(self._ssh_cmd[0], self._ssh_cmd[1:])


@plugin_required
@plugin_version("1.1.23.0")
@valid_aws_profile
@valid_aws_region
def ssh(
    config,
    instance_name,
    user=DEFAULT_OS_USER,
    port=DEFAULT_SSH_PORT,
    key_type=DEFAULT_KEY_ALGORITHM,
    key_size=DEFAULT_KEY_SIZE,
    profile_name=AWS_DEFAULT_PROFILE,
    region_name=AWS_DEFAULT_REGION,
    command=None,
):
    instance, profile, region = fetch_instance_details_from_config(
        config, instance_name, profile_name, region_name
    )

    ssm = get_aws_client("ssm", region_name=region, profile_name=profile)
    ec2 = get_aws_resource("ec2", region_name=region, profile_name=profile)
    ec2_ic = get_aws_client(
        "ec2-instance-connect", region_name=region, profile_name=profile
    )

    instance_id = query_instance(name=instance, ec2=ec2)
    if instance_id is None:
        raise ValueError("No instance could be found for name: {}".format(instance))

    az = get_instance_details(instance_id=instance_id, ec2=ec2)["availability_zone"]

    logger.info(
        "Opening SSH session on instance %s (%s) via profile %s",
        instance_id,
        region,
        profile,
    )
    with SshKey(key_type=key_type, key_size=key_size) as ssh_key:
        with SshKeyUploader(
            instance_id=instance_id, az=az, user=user, ssh_key=ssh_key, ec2_ic=ec2_ic
        ):
            with SshSession(
                instance_id,
                region_name=region,
                profile_name=profile,
                ssm=ssm,
                port=port,
                user=user,
                command=command,
            ) as ssh_session:
                ssh_session.open()



import csv
import io
import json
import logging

from aws_gate.constants import (
    AWS_DEFAULT_PROFILE,
    AWS_DEFAULT_REGION,
    DEFAULT_LIST_OUTPUT_FIELDS,
    DEFAULT_LIST_HUMAN_FIELDS,
    DEFAULT_LIST_OUTPUT,
)
from aws_gate.decorators import valid_aws_region, valid_aws_profile
from aws_gate.utils import get_aws_client, get_aws_resource, get_instance_details

logger = logging.getLogger(__name__)


# pylint: disable=unused-argument
def _serialize_json(data, fields=None):
    return json.dumps(data, indent=4, sort_keys=True)


def _serialize_csv(data, delimiter=",", fields=DEFAULT_LIST_OUTPUT_FIELDS):
    output = io.StringIO()
    writer = csv.DictWriter(output, delimiter=delimiter, fieldnames=fields)
    writer.writerows(data)

    return output.getvalue()


def _serialize_tsv(data, fields=DEFAULT_LIST_OUTPUT_FIELDS):
    return _serialize_csv(data, delimiter="\t", fields=fields)


def _serialize_human(data, fields=DEFAULT_LIST_HUMAN_FIELDS):
    return _serialize_csv(data, delimiter=" ", fields=fields)


def serialize(
    data, output_format=DEFAULT_LIST_OUTPUT, fields=DEFAULT_LIST_OUTPUT_FIELDS
):
    format_dispatcher = {
        "csv": _serialize_csv,
        "tsv": _serialize_tsv,
        "human": _serialize_human,
        "json": _serialize_json,
    }

    filtered_data = []
    for item in data:
        filtered_data_item = {}
        for field in fields:
            if field in item.keys():
                filtered_data_item[field] = item[field]
        filtered_data.append(filtered_data_item)

    return format_dispatcher[output_format](filtered_data, fields=fields)


@valid_aws_profile
@valid_aws_region
def list_instances(
    profile_name=AWS_DEFAULT_PROFILE,
    region_name=AWS_DEFAULT_REGION,
    output_format=DEFAULT_LIST_OUTPUT,
    fields=DEFAULT_LIST_HUMAN_FIELDS,
):
    invalid_fields = list(set(fields) - set(DEFAULT_LIST_OUTPUT_FIELDS))
    if invalid_fields:
        raise ValueError(
            'Invalid fields provided: "{}". Valid fields: "{}"'.format(
                " ".join(invalid_fields), " ".join(DEFAULT_LIST_OUTPUT_FIELDS)
            )
        )

    ssm = get_aws_client("ssm", region_name=region_name, profile_name=profile_name)
    ec2 = get_aws_resource("ec2", region_name=region_name, profile_name=profile_name)

    instances_ssm_paginator = ssm.get_paginator("describe_instance_information")
    instances_ssm_response_iterator = instances_ssm_paginator.paginate()

    instance_ids = []
    for response in instances_ssm_response_iterator:
        for instance in response["InstanceInformationList"]:
            instance_ids.append(instance["InstanceId"])

    instance_details = []
    for instance_id in instance_ids:
        instance_details.append(get_instance_details(instance_id=instance_id, ec2=ec2))

    print(
        serialize(instance_details, output_format=output_format, fields=fields).rstrip()
    )



import logging
import os

from wrapt import decorator

from aws_gate.constants import PLUGIN_INSTALL_PATH, PLUGIN_NAME
from aws_gate.utils import execute_plugin, is_existing_profile, is_existing_region

logger = logging.getLogger(__name__)


def _plugin_exists(plugin_path):
    return os.path.exists(plugin_path)


def _plugin_exists_in_path():
    return any(
        _plugin_exists(os.path.join(path, PLUGIN_NAME))
        for path in os.environ["PATH"].split(os.path.pathsep)
    )


@decorator
def plugin_required(
    wrapped_function, instance, args, kwargs
):  # pylint: disable=unused-argument
    if not _plugin_exists(PLUGIN_INSTALL_PATH) and not _plugin_exists_in_path():
        raise OSError("{} not found".format(PLUGIN_NAME))

    return wrapped_function(*args, **kwargs)


def plugin_version(required_version):
    @decorator
    def wrapper(
        wrapped_function, instance, args, kwargs
    ):  # pylint: disable=unused-argument
        version = execute_plugin(["--version"], capture_output=True)
        logger.debug(
            "session-manager-plugin version: %s (required version: %s)",
            version,
            required_version,
        )

        if version and int(version.replace(".", "")) < int(
            required_version.replace(".", "")
        ):
            raise ValueError("Invalid plugin version: {}".format(version))

        return wrapped_function(*args, **kwargs)

    return wrapper


@decorator
def valid_aws_profile(
    wrapped_function, instance, args, kwargs
):  # pylint: disable=unused-argument
    if not is_existing_profile(kwargs["profile_name"]):
        raise ValueError("Invalid profile provided: {}".format(kwargs["profile_name"]))

    return wrapped_function(*args, **kwargs)


@decorator
def valid_aws_region(
    wrapped_function, instance, args, kwargs
):  # pylint: disable=unused-argument
    if not is_existing_region(kwargs["region_name"]):
        raise ValueError("Invalid region provided: {}".format(kwargs["region_name"]))

    return wrapped_function(*args, **kwargs)



import pytest

from aws_gate.constants import DEFAULT_GATE_KEY_PATH
from aws_gate.ssh_config import ssh_config, PROXY_COMMAND


def test_ssh_config(mocker, capsys):
    expected_output_lines = [
        """Host *.eu-west-1.default""",
        """IdentityFile {}""".format(DEFAULT_GATE_KEY_PATH),
        """IdentitiesOnly yes""",
        """User ec2-user""",
        """Port 22""",
        """ProxyCommand {}""".format(" ".join(PROXY_COMMAND)),
        "\n",
    ]
    expected_output = "\n".join(expected_output_lines)

    mocker.patch("aws_gate.decorators.is_existing_region", return_value=True)
    mocker.patch("aws_gate.decorators.is_existing_profile", return_value=True)

    ssh_config(profile_name="default", region_name="eu-west-1")

    out, _ = capsys.readouterr()

    assert out == expected_output


def test_ssh_config_invalid_profile(mocker):
    mocker.patch("aws_gate.decorators.is_existing_region", return_value=True)
    with pytest.raises(ValueError):
        ssh_config(profile_name="invalid-profile", region_name="eu-west-1")


def test_ssh_config_invalid_region(mocker):
    mocker.patch("aws_gate.decorators.is_existing_profile", return_value=True)
    with pytest.raises(ValueError):
        ssh_config(profile_name="default", region_name="invalid-region")



import json
import logging

from aws_gate.utils import execute_plugin

logger = logging.getLogger(__name__)


class BaseSession:
    _instance_id = None
    _ssm = None
    _region_name = None
    _profile_name = None

    _response = None
    _session_id = None
    _token_value = None

    _session_parameters = {}

    def __enter__(self):
        # create and establish session
        self.create()
        return self

    def __exit__(self, *args):
        # terminate session
        self.terminate()

    def create(self):
        logger.debug(
            "Creating a new session on instance: %s (%s)",
            self._instance_id,
            self._region_name,
        )
        self._response = self._ssm.start_session(**self._session_parameters)
        logger.debug("Received response: %s", self._response)

        self._session_id, self._token_value = (
            self._response["SessionId"],
            self._response["TokenValue"],
        )

    def terminate(self):
        logger.debug("Terminating session: %s", self._session_id)
        response = self._ssm.terminate_session(SessionId=self._session_id)
        logger.debug("Received response: %s", response)

    def open(self):
        execute_plugin(
            [
                json.dumps(self._response),
                self._region_name,
                "StartSession",
                self._profile_name,
                json.dumps(self._session_parameters),
                self._ssm.meta.endpoint_url,
            ]
        )



# pylint: disable=wrong-import-position
import pytest

from aws_gate.session import SSMSession, session  # noqa


def test_create_ssm_session(ssm_mock, instance_id):
    sess = SSMSession(instance_id=instance_id, ssm=ssm_mock)
    sess.create()

    assert ssm_mock.start_session.called


def test_terminate_ssm_session(ssm_mock, instance_id):
    sess = SSMSession(instance_id=instance_id, ssm=ssm_mock)

    sess.create()
    sess.terminate()

    assert ssm_mock.terminate_session.called


def test_open_ssm_session(mocker, ssm_mock, instance_id):
    m = mocker.patch("aws_gate.session_common.execute_plugin", return_value="output")
    sess = SSMSession(instance_id=instance_id, ssm=ssm_mock)
    sess.open()

    assert m.called


def test_ssm_session_context_manager(ssm_mock, instance_id):
    with SSMSession(instance_id=instance_id, ssm=ssm_mock):
        pass

    assert ssm_mock.start_session.called
    assert ssm_mock.terminate_session.called


def test_ssm_session(mocker, instance_id, config):
    mocker.patch("aws_gate.session.get_aws_client")
    mocker.patch("aws_gate.session.get_aws_resource")
    mocker.patch("aws_gate.session.query_instance", return_value=instance_id)
    session_mock = mocker.patch(
        "aws_gate.session.SSMSession", return_value=mocker.MagicMock()
    )
    mocker.patch("aws_gate.decorators._plugin_exists", return_value=True)
    mocker.patch("aws_gate.decorators.execute_plugin", return_value="1.1.23.0")
    mocker.patch("aws_gate.decorators.is_existing_profile", return_value=True)

    session(
        config=config,
        instance_name=instance_id,
        profile_name="profile",
        region_name="eu-west-1",
    )
    assert session_mock.called


def test_ssm_session_exception_invalid_profile(mocker, instance_id, config):
    mocker.patch("aws_gate.session.get_aws_client")
    mocker.patch("aws_gate.session.get_aws_resource")
    mocker.patch("aws_gate.session.query_instance", return_value=None)
    mocker.patch("aws_gate.decorators._plugin_exists", return_value=True)
    mocker.patch("aws_gate.decorators.execute_plugin", return_value="1.1.23.0")

    with pytest.raises(ValueError):
        session(
            config=config, profile_name="invalid-profile", instance_name=instance_id
        )


def test_ssm_session_exception_invalid_region(mocker, instance_id, config):
    mocker.patch("aws_gate.session.get_aws_client")
    mocker.patch("aws_gate.session.get_aws_resource")
    mocker.patch("aws_gate.session.query_instance", return_value=None)
    mocker.patch("aws_gate.decorators._plugin_exists", return_value=True)
    mocker.patch("aws_gate.decorators.execute_plugin", return_value="1.1.23.0")

    with pytest.raises(ValueError):
        session(
            config=config,
            region_name="invalid-region",
            instance_name=instance_id,
            profile_name="default",
        )


def test_ssm_session_exception_unknown_instance_id(mocker, instance_id, config):
    mocker.patch("aws_gate.session.get_aws_client")
    mocker.patch("aws_gate.session.get_aws_resource")
    mocker.patch("aws_gate.session.query_instance", return_value=None)
    mocker.patch("aws_gate.decorators._plugin_exists", return_value=True)
    mocker.patch("aws_gate.decorators.execute_plugin", return_value="1.1.23.0")
    mocker.patch("aws_gate.decorators.is_existing_profile", return_value=True)

    with pytest.raises(ValueError):
        session(
            config=config,
            instance_name=instance_id,
            profile_name="profile",
            region_name="eu-west-1",
        )


def test_ssm_session_without_config(mocker, instance_id, empty_config):
    mocker.patch("aws_gate.session.get_aws_client")
    mocker.patch("aws_gate.session.get_aws_resource")
    mocker.patch("aws_gate.session.query_instance", return_value=None)
    mocker.patch("aws_gate.decorators._plugin_exists", return_value=True)
    mocker.patch("aws_gate.decorators.execute_plugin", return_value="1.1.23.0")

    with pytest.raises(ValueError):
        session(
            config=empty_config,
            instance_name=instance_id,
            profile_name="profile",
            region_name="eu-west-1",
        )



import argparse
import os
from unittest.mock import MagicMock, create_autospec

import pytest
from marshmallow import ValidationError

from aws_gate.cli import main, _get_profile, _get_region, parse_arguments


def test_cli_param_error():
    with pytest.raises(SystemExit):
        main()


def test_cli_invalid_config(mocker):
    mocker.patch(
        "aws_gate.cli.parse_arguments",
        return_value=mocker.MagicMock(subcommand="bootstrap"),
    )
    mocker.patch(
        "aws_gate.cli.load_config_from_files",
        side_effect=ValidationError(message="error"),
    )

    with pytest.raises(ValueError):
        main()


def test_cli_parse_arguments_unknown_subcommand(mocker):
    parser_mock = mocker.MagicMock()
    parser_mock.configure_mock(
        **{"parse_args.return_value": mocker.MagicMock(subcommand=False)}
    )

    exit_mock = mocker.patch("sys.exit")
    mocker.patch("aws_gate.cli.argparse.ArgumentParser", return_value=parser_mock)

    parse_arguments()

    assert parser_mock.print_help.called
    assert exit_mock.called
    assert exit_mock.call_args == mocker.call(1)


def test_cli_default_profile_from_aws_vault(mocker):
    mocker.patch.dict(os.environ, {"AWS_VAULT": "vault_profile"})
    mocker.patch(
        "aws_gate.cli.parse_arguments", return_value=mocker.MagicMock(subcommand="list")
    )
    mocker.patch("aws_gate.decorators.is_existing_region", return_value=True)
    mocker.patch("aws_gate.decorators.is_existing_profile", return_value=True)
    mocker.patch("aws_gate.cli.list_instances")
    logger_mock = mocker.patch("aws_gate.cli.logging.getLogger")
    m = mocker.patch("aws_gate.cli._get_profile")

    main()

    assert m.called
    assert m.call_args[1]["default"] == "vault_profile"
    assert logger_mock.called


@pytest.mark.parametrize(
    "args, config, default, expected",
    [
        (
            MagicMock(profile="args_profile", region="args_region"),
            MagicMock(default_profile="config_profile", default_region="config_region"),
            "args_profile",
            "args_profile",
        ),
        (
            create_autospec(argparse.Namespace),
            MagicMock(default_profile="config_profile", default_region="config_region"),
            "config_profile",
            "config_profile",
        ),
        (
            create_autospec(argparse.Namespace),
            MagicMock(default_profile=None),
            "default_profile",
            "default_profile",
        ),
    ],
    ids=["args", "config", "default"],
)
def test_cli_get_profile(args, config, default, expected):
    assert _get_profile(args, config, default) == expected


@pytest.mark.parametrize(
    "args, config, default, expected",
    [
        (
            MagicMock(profile="args_profile", region="args_region"),
            MagicMock(default_profile="config_profile", default_region="config_region"),
            "args_region",
            "args_region",
        ),
        (
            create_autospec(argparse.Namespace),
            MagicMock(default_profile="config_profile", default_region="config_region"),
            "config_region",
            "config_region",
        ),
        (
            create_autospec(argparse.Namespace),
            MagicMock(default_region=None),
            "default_region",
            "default_region",
        ),
    ],
    ids=["args", "config", "default"],
)
def test_cli_get_region(args, config, default, expected):
    assert _get_region(args, config, default) == expected


@pytest.mark.parametrize(
    "subcommand",
    [
        ("bootstrap", "bootstrap"),
        ("list", "list_instances"),
        ("ls", "list_instances"),
        ("session", "session"),
        ("ssh", "ssh"),
        ("ssh-config", "ssh_config"),
        ("ssh-proxy", "ssh_proxy"),
    ],
    ids=lambda x: x[0],
)
def test_cli_subcommand(mocker, subcommand):
    mocker.patch(
        "aws_gate.cli.parse_arguments",
        return_value=mocker.MagicMock(subcommand=subcommand[0]),
    )
    m = mocker.patch("aws_gate.cli.{}".format(subcommand[1]))

    main()

    assert m.called



#!/usr/bin/env python

import argparse
import multiprocessing
import os
import sys

parser = argparse.ArgumentParser()
parser.add_argument(
    "-L",
    "--no-lint",
    action="store_false",
    default=True,
    dest="lint",
    help="Skip file linting",
)
parser.add_argument(
    "-C",
    "--no-coverage",
    action="store_false",
    default=True,
    dest="coverage",
    help="Skip coverage report",
)
args, rest = parser.parse_known_args()

root_dir = os.path.split(os.path.dirname(sys.argv[0]))[0]
report_dir = os.path.join(root_dir, "reports")

if not os.path.exists(report_dir):
    os.mkdir(report_dir, mode=0o755)

if "PLACEBO_MODE" not in os.environ:
    os.environ["PLACEBO_MODE"] = "playback"

os.environ["PLACEBO_DIR"] = os.path.join(root_dir, "tests", "unit", "placebo")

cmd = ["pytest", "--verbose"]

if args.lint:
    lint_args = [
        "--flake8",
        "--pylint",
        "--pylint-jobs={}".format(multiprocessing.cpu_count()),
        "--pylint-rcfile={}".format(os.path.join(root_dir, ".pylintrc")),
    ]
    cmd.extend(lint_args)

if args.coverage:
    coverage_args = [
        "--cov=aws_gate",
        "--cov-report=term-missing",
        "--cov-report=xml:reports/coverage.xml",
        "--cov-report=html:reports/coverage/aws-gate",
        "--junitxml=reports/test.xml",
    ]
    cmd.extend(coverage_args)
else:
    cmd.append("--no-cov")

cmd.extend(rest)

os.execvp(cmd[0], cmd)



import contextlib
import errno
import logging
import os
import signal
import subprocess

import boto3
import botocore

from aws_gate.constants import DEFAULT_GATE_BIN_PATH, PLUGIN_NAME
from aws_gate.exceptions import AWSConnectionError

logger = logging.getLogger(__name__)

# This list is maintained by hand as new regions are not added that often. This should be
# removed once, we find a better way how to obtain region list without the need to
# contact AWS EC2 API
AWS_REGIONS = [
    "ap-northeast-1",
    "ap-northeast-2",
    "ap-northeast-3",
    "ap-south-1",
    "ap-southeast-1",
    "ap-southeast-2",
    "ca-central-1",
    "eu-central-1",
    "eu-north-1",
    "eu-west-1",
    "eu-west-2",
    "eu-west-3",
    "sa-east-1",
    "us-east-1",
    "us-east-2",
    "us-west-1",
    "us-west-2",
]


def _create_aws_session(region_name=None, profile_name=None):
    logger.debug("Obtaining boto3 session object")
    kwargs = {}
    if region_name is not None:
        kwargs["region_name"] = region_name
    if profile_name is not None:
        kwargs["profile_name"] = profile_name

    # https://github.com/boto/boto3/issues/598
    if "AWS_ACCESS_KEY_ID" in os.environ:
        kwargs["aws_access_key_id"] = os.environ["AWS_ACCESS_KEY_ID"]
    if "AWS_SECRET_ACCESS_KEY" in os.environ:
        kwargs["aws_secret_access_key"] = os.environ["AWS_SECRET_ACCESS_KEY"]
    if "AWS_SESSION_TOKEN" in os.environ:
        kwargs["aws_session_token"] = os.environ["AWS_SESSION_TOKEN"]

    session = boto3.session.Session(**kwargs)

    return session


def get_aws_client(service_name, region_name, profile_name=None):
    session = _create_aws_session(region_name=region_name, profile_name=profile_name)

    logger.debug("Obtaining %s client", service_name)
    return session.client(service_name=service_name)


def get_aws_resource(service_name, region_name, profile_name=None):
    session = _create_aws_session(region_name=region_name, profile_name=profile_name)

    logger.debug("Obtaining %s boto3 resource", service_name)
    return session.resource(service_name=service_name)


def is_existing_profile(profile_name):
    session = _create_aws_session()

    logger.debug(
        "Obtained configured AWS profiles: %s", " ".join(session.available_profiles)
    )
    return profile_name in session.available_profiles


def is_existing_region(region_name):
    return region_name in AWS_REGIONS


def get_default_region():
    session = _create_aws_session()

    return session.region_name


@contextlib.contextmanager
def deferred_signals(signal_list=None):
    if signal_list is None:
        signal_list = [signal.SIGHUP, signal.SIGINT, signal.SIGTERM]

    for deferred_signal in signal_list:
        signal_name = signal.Signals(deferred_signal).name
        logger.debug("Deferring signal: %s", signal_name)
        signal.signal(deferred_signal, signal.SIG_IGN)

    try:
        yield
    finally:
        for deferred_signal in signal_list:
            signal_name = signal.Signals(deferred_signal).name
            logger.debug("Restoring signal: %s", signal_name)
            signal.signal(deferred_signal, signal.SIG_DFL)


def execute(cmd, args, **kwargs):
    ret, result = None, None

    env = DEFAULT_GATE_BIN_PATH + os.pathsep + os.environ["PATH"]

    try:
        logger.debug('Executing "%s"', " ".join([cmd] + args))
        result = subprocess.run([cmd] + args, env={"PATH": env}, check=True, **kwargs)
    except subprocess.CalledProcessError as e:
        logger.error(
            'Command "%s" exited with %s', " ".join([cmd] + args), e.returncode
        )
    except OSError as e:
        if e.errno == errno.ENOENT:
            raise ValueError("{} cannot be found".format(cmd))

    if result and result.stdout:
        ret = result.stdout.decode()
        ret = ret.rstrip()

    return ret


def execute_plugin(args, **kwargs):
    with deferred_signals():
        return execute(PLUGIN_NAME, args, **kwargs)


def fetch_instance_details_from_config(
    config, instance_name, profile_name, region_name
):
    config_data = config.get_host(instance_name)
    if (
        config_data
        and config_data["name"]
        and config_data["profile"]
        and config_data["region"]
    ):
        region = config_data["region"]
        profile = config_data["profile"]
        instance = config_data["name"]
    else:
        region = region_name
        profile = profile_name
        instance = instance_name

    return instance, profile, region


def get_instance_details(instance_id, ec2=None):
    filters = [{"Name": "instance-id", "Values": [instance_id]}]

    try:
        ec2_instance = list(ec2.instances.filter(Filters=filters))[0]
    except botocore.exceptions.ClientError:
        raise AWSConnectionError

    instance_name = None
    for tag in ec2_instance.tags:
        if tag["Key"] == "Name":
            instance_name = tag["Value"]

    return {
        "instance_id": instance_id,
        "instance_name": instance_name,
        "availability_zone": ec2_instance.placement["AvailabilityZone"],
        "vpc_id": ec2_instance.vpc_id,
        "private_ip_address": ec2_instance.private_ip_address or None,
        "public_ip_addess": ec2_instance.public_ip_address or None,
        "private_dns_name": ec2_instance.private_dns_name or None,
        "public_dns_name": ec2_instance.public_dns_name or None,
    }



import os

import boto3
import placebo
import pytest


@pytest.fixture(name="session")
def placebo_session(request):
    session_kwargs = {"region_name": os.environ.get("AWS_DEFAULT_REGION", "eu-west-1")}
    profile_name = os.environ.get("PLACEBO_PROFILE", None)
    if profile_name:
        session_kwargs["profile_name"] = profile_name

    session = boto3.Session(**session_kwargs)

    prefix = request.function.__name__

    base_dir = os.environ.get("PLACEBO_DIR", os.path.join(os.getcwd(), "placebo"))
    record_dir = os.path.join(base_dir, prefix)

    if not os.path.exists(record_dir):
        os.makedirs(record_dir)

    pill = placebo.attach(session, data_path=record_dir)

    if os.environ.get("PLACEBO_MODE") == "record":
        pill.record()
    else:
        pill.playback()

    return session


@pytest.fixture
def ec2(session):
    return session.resource("ec2", region_name="eu-west-1")


@pytest.fixture
def ec2_ic(session):
    return session.resource("ec2-instance-connect", region_name="eu-west-1")


@pytest.fixture
def ssm(session):
    return session.client("ssm", region_name="eu-west-1")


@pytest.fixture
def ec2_mock(mocker):
    return mocker.MagicMock()


@pytest.fixture
def ec2_ic_mock(mocker):
    return mocker.MagicMock()


@pytest.fixture
def ssm_mock(mocker):
    mock = mocker.MagicMock()
    response = {
        "SessionId": "session-020bf6cd31f912b53",
        "TokenValue": "randomtokenvalue",
    }
    mock.configure_mock(
        **{
            "start_session.return_value": response,
            "terminate_session.return_value": response,
        }
    )
    type(mock.meta).endpoint_url = mocker.PropertyMock(return_value="ssm")
    return mock


@pytest.fixture
def instance_id():
    return "i-0c32153096cd68a6d"


@pytest.fixture
def ssh_key(mocker):
    mock = mocker.MagicMock()
    mock.configure_mock(
        **{
            "public_key.return_value": "ssh-rsa ranodombase64string",
            "key_path.return_value": "/home/user/.aws-gate/key",
        }
    )

    return mock


@pytest.fixture
def config(mocker):
    mock = mocker.MagicMock()
    mock.configure_mock(
        **{
            "get_host.return_value": {
                "alias": "test",
                "name": "SSM-test",
                "profile": "default",
                "region": "eu-west-1",
            }
        }
    )

    return mock


@pytest.fixture
def empty_config(mocker):
    mock = mocker.MagicMock()
    mock.configure_mock(**{"get_host.return_value": {}})

    return mock


@pytest.fixture
def get_instance_details_response():
    return {"availability_zone": "eu-west-1a"}



import logging
from collections import OrderedDict

from aws_gate.constants import (
    AWS_DEFAULT_PROFILE,
    AWS_DEFAULT_REGION,
    DEFAULT_SSH_PORT,
    DEFAULT_OS_USER,
    DEFAULT_GATE_KEY_PATH,
)
from aws_gate.decorators import valid_aws_region, valid_aws_profile

logger = logging.getLogger(__name__)

PROXY_COMMAND = [
    r"""sh -c "aws-gate ssh-proxy -p `echo %h | sed -Ee 's/^(.*)\.(.*)\.(.*)$/\\3/g'`""",
    r"""-r `echo %h | sed -Ee 's/^(.*)\.(.*)\.(.*)$/\\2/g'`""",
    r'''`echo %h | sed -Ee 's/^(.*)\.(.*)\.(.*)$/\\1/g'`"''',
]


@valid_aws_profile
@valid_aws_region
def ssh_config(
    profile_name=AWS_DEFAULT_PROFILE,
    region_name=AWS_DEFAULT_REGION,
    user=DEFAULT_OS_USER,
    port=DEFAULT_SSH_PORT,
):
    config = OrderedDict(
        {
            "Host": "*.{}.{}".format(region_name, profile_name),
            "IdentityFile": DEFAULT_GATE_KEY_PATH,
            "IdentitiesOnly": "yes",
            "User": user,
            "Port": port,
            "ProxyCommand": " ".join(PROXY_COMMAND),
        }
    )
    for k, v in config.items():
        print("{} {}".format(k, v))
    print()



import logging
import os

import yaml
from marshmallow import Schema, fields, post_load, ValidationError
from yaml.constructor import ConstructorError
from yaml.parser import ParserError

from aws_gate.constants import DEFAULT_GATE_CONFIG_PATH, DEFAULT_GATE_CONFIGD_PATH
from aws_gate.utils import is_existing_profile, is_existing_region

logger = logging.getLogger(__name__)


class EmptyConfigurationError(Exception):
    pass


def validate_profile(profile):
    if not is_existing_profile(profile):
        raise ValidationError("Invalid profile provided: {}".format(profile))


def validate_region(region):
    if not is_existing_region(region):
        raise ValidationError("Invalid region name provided: {}".format(region))


def validate_defaults(data):
    schema = DefaultsSchema()
    schema.load(data)


class DefaultsSchema(Schema):
    profile = fields.String(required=False, validate=validate_profile)
    region = fields.String(required=False, validate=validate_region)


class HostSchema(Schema):
    alias = fields.String(required=True)
    name = fields.String(required=True)
    profile = fields.String(required=True, validate=validate_profile)
    region = fields.String(required=True, validate=validate_region)


class GateConfigSchema(Schema):
    defaults = fields.Nested(
        DefaultsSchema, required=False, missing=dict(), validate=validate_defaults
    )
    hosts = fields.List(fields.Nested(HostSchema), required=False, missing=list())

    # pylint: disable=no-self-use,unused-argument
    @post_load
    def create_config(self, data, **kwargs):
        return GateConfig(**data)


class GateConfig:
    def __init__(self, defaults, hosts):
        self._defaults = defaults
        self._hosts = hosts

    @property
    def hosts(self):
        return self._hosts

    @property
    def defaults(self):
        return self._defaults

    @property
    def default_region(self):
        if "region" in self._defaults:
            return self._defaults["region"]
        return None

    @property
    def default_profile(self):
        if "profile" in self._defaults:
            return self._defaults["profile"]
        return None

    def get_host(self, name):
        host = [host for host in self._hosts if host["alias"] == name]
        if host:
            return host[0]
        return {}


def _locate_config_files():
    config_files = []

    if os.path.isdir(DEFAULT_GATE_CONFIGD_PATH):
        configd_files = sorted(os.listdir(DEFAULT_GATE_CONFIGD_PATH))
        for f in configd_files:
            file_path = os.path.join(DEFAULT_GATE_CONFIGD_PATH, f)
            if os.path.isfile(file_path):
                logger.debug("Located config file: %s", file_path)
                config_files.append(file_path)

    if os.path.isfile(DEFAULT_GATE_CONFIG_PATH):
        logger.debug("Located config file: %s", DEFAULT_GATE_CONFIG_PATH)
        config_files.append(DEFAULT_GATE_CONFIG_PATH)

    return config_files


def _merge_data(src, dst):
    if isinstance(dst, dict):
        if isinstance(src, dict):
            for key in src:
                if key in dst:
                    dst[key] = _merge_data(src[key], dst[key])
                else:
                    dst[key] = src[key]
        else:
            raise TypeError(
                "Cannot merge {} with dict, src={} dst={}".format(
                    type(src).__name__, src, dst
                )
            )

    elif isinstance(dst, list):
        if isinstance(src, list):
            dst.extend(src)
        else:
            dst.append(src)
    else:
        dst = src

    return dst


def load_config_from_files(config_files=None):
    if config_files is None:
        config_files = _locate_config_files()

    config_data, data = {}, {}
    if config_files:
        for path in config_files:
            try:
                with open(path, "r") as config_file:
                    data = yaml.safe_load(config_file) or {}
            except (ConstructorError, ParserError):
                data = {}
            _merge_data(data, config_data)

        if not config_data:
            raise EmptyConfigurationError("Empty configuration data")

    config = GateConfigSchema().load(config_data)
    return config



import errno
import os
import subprocess

import pytest
from botocore.exceptions import ClientError
from hypothesis import given
from hypothesis.strategies import lists, text

from aws_gate.exceptions import AWSConnectionError
from aws_gate.utils import (
    is_existing_profile,
    _create_aws_session,
    get_aws_client,
    get_aws_resource,
    AWS_REGIONS,
    is_existing_region,
    execute,
    execute_plugin,
    fetch_instance_details_from_config,
    get_instance_details,
)


# pylint: disable=too-few-public-methods
class MockSession:
    def __init__(self):
        self._available_profiles = ["profile{}".format(i) for i in range(5)]

    @property
    def available_profiles(self):
        return self._available_profiles


def test_existing_profile(mocker):
    mocker.patch("aws_gate.utils._create_aws_session", return_value=MockSession())

    assert is_existing_profile("profile1")
    assert not is_existing_profile("nonexistentprofile")


def test_create_aws_session(mocker):
    session_mock = mocker.patch(
        "aws_gate.utils.boto3.session", return_value=mocker.MagicMock()
    )

    _create_aws_session(region_name="eu-west-1")

    assert session_mock.Session.called
    assert session_mock.Session.call_args == mocker.call(region_name="eu-west-1")


def test_create_aws_session_with_profile(mocker):
    session_mock = mocker.patch(
        "aws_gate.utils.boto3.session", return_value=mocker.MagicMock()
    )
    _create_aws_session(region_name="eu-west-1", profile_name="default")

    assert session_mock.Session.called
    assert session_mock.Session.call_args == mocker.call(
        region_name="eu-west-1", profile_name="default"
    )


def test_create_aws_profile_credentials_from_env_vars(mocker):
    credentials_dict = {
        "AWS_ACCESS_KEY_ID": "a",
        "AWS_SECRET_ACCESS_KEY": "b",
        "AWS_SESSION_TOKEN": "c",
    }
    session_mock = mocker.patch(
        "aws_gate.utils.boto3.session", return_value=mocker.MagicMock()
    )
    mocker.patch.dict(os.environ, credentials_dict)

    _create_aws_session()

    assert session_mock.Session.called
    assert session_mock.Session.call_args == mocker.call(
        aws_access_key_id="a", aws_secret_access_key="b", aws_session_token="c"
    )


def test_get_aws_client(mocker):
    mock = mocker.patch(
        "aws_gate.utils._create_aws_session", return_value=mocker.MagicMock()
    )

    get_aws_client(service_name="ec2", region_name="eu-west-1")

    assert mock.called
    assert mock.mock_calls == [mocker.call(profile_name=None, region_name="eu-west-1")]


def test_get_aws_resource(mocker):
    mock = mocker.patch(
        "aws_gate.utils._create_aws_session", return_value=mocker.MagicMock()
    )
    get_aws_resource(service_name="ec2", region_name="eu-west-1")

    assert mock.called


def test_region_validation():
    assert is_existing_region(region_name=AWS_REGIONS[0])
    assert not is_existing_region(region_name="unknown-region-1")


@given(text(), lists(text()))
def test_execute(mocker, cmd, args):
    mock_output = mocker.MagicMock(stdout=b"output")
    mocker.patch("aws_gate.utils.subprocess.run", return_value=mock_output)

    assert execute(cmd, args) == "output"


def test_execute_command_exited_with_nonzero_rc(mocker):
    mock = mocker.patch(
        "aws_gate.utils.subprocess.run",
        side_effect=subprocess.CalledProcessError(returncode=1, cmd="error"),
    )
    execute("/usr/bin/ls", ["-l"])

    assert mock.called


def test_execute_command_not_found(mocker):
    mocker.patch(
        "aws_gate.utils.subprocess.run",
        side_effect=OSError(errno.ENOENT, os.strerror(errno.ENOENT)),
    )
    with pytest.raises(ValueError):
        execute("/usr/bin/ls", ["-l"])


def test_execute_plugin(mocker):
    mocker.patch("aws_gate.utils.execute", return_value="output")
    output = execute_plugin(["--version"], capture_output=True)

    assert output == "output"


def test_execute_plugin_args(mocker):
    m = mocker.patch("aws_gate.utils.execute", return_value="output")

    execute_plugin(["--version"], capture_output=True)

    assert m.called
    assert "['--version'], capture_output=True" in str(m.call_args)


def test_fetch_instance_details_from_config(config):
    expected_instance_name = config.get_host()["name"]
    expected_profile = config.get_host()["profile"]
    expeted_region = config.get_host()["region"]

    instance_name, profile, region = fetch_instance_details_from_config(
        config, "instance_name", "profile", "region"
    )

    assert expected_instance_name == instance_name
    assert expected_profile == profile
    assert expeted_region == region


def test_fetch_instance_details_from_config_with_empty_config(empty_config):
    expected_instance_name = "instance_name"
    expected_profile = "profile"
    expeted_region = "region"

    instance_name, profile, region = fetch_instance_details_from_config(
        empty_config, expected_instance_name, expected_profile, expeted_region
    )

    assert expected_instance_name == instance_name
    assert expected_profile == profile
    assert expeted_region == region


def test_get_instance_details_aws_api_exception(ec2_mock, instance_id):
    # https://github.com/surbas/pg2kinesis/blob/master/tests/test_stream.py#L20
    error_response = {"Error": {"Code": "ResourceInUseException"}}
    ec2_mock.configure_mock(
        **{"instances.filter.side_effect": ClientError(error_response, "random_ec2_op")}
    )

    with pytest.raises(AWSConnectionError):
        get_instance_details(instance_id, ec2=ec2_mock)


def test_get_instance_details(instance_id, ec2):
    expected_details = {
        "instance_id": "i-0c32153096cd68a6d",
        "vpc_id": "vpc-1981f29759da4a354",
        "private_dns_name": "ip-10-69-104-49.eu-west-1.compute.internal",
        "private_ip_address": "10.69.104.49",
        "public_dns_name": "ec2-18-201-115-108.eu-west-1.compute.amazonaws.com",
        "public_ip_addess": "18.201.115.108",
        "availability_zone": "eu-west-1a",
        "instance_name": "dummy-instance",
    }

    details = get_instance_details(instance_id, ec2=ec2)

    assert details == expected_details



import argparse
import logging
import os
import sys

from marshmallow import ValidationError
from yaml.scanner import ScannerError

from aws_gate import __version__, __description__
from aws_gate.bootstrap import bootstrap
from aws_gate.config import load_config_from_files
from aws_gate.constants import (
    SUPPORTED_KEY_TYPES,
    DEBUG,
    AWS_DEFAULT_REGION,
    AWS_DEFAULT_PROFILE,
    DEFAULT_OS_USER,
    DEFAULT_SSH_PORT,
    DEFAULT_KEY_ALGORITHM,
    DEFAULT_KEY_SIZE,
    DEFAULT_LIST_HUMAN_FIELDS,
    DEFAULT_LIST_OUTPUT_FORMATS,
    DEFAULT_LIST_OUTPUT,
)
from aws_gate.list import list_instances
from aws_gate.session import session
from aws_gate.ssh import ssh
from aws_gate.ssh_config import ssh_config
from aws_gate.ssh_proxy import ssh_proxy
from aws_gate.utils import get_default_region

logger = logging.getLogger(__name__)


def _get_profile(args, config, default):
    profile = None
    if hasattr(args, "profile"):
        profile = args.profile
    return profile or config.default_profile or default


def _get_region(args, config, default):
    region = None
    if hasattr(args, "region"):
        region = args.region
    return region or config.default_region or default


def parse_arguments():
    parser = argparse.ArgumentParser(description=__description__)
    parser.add_argument(
        "-v", "--verbose", help="increase output verbosity", action="store_true"
    )
    parser.add_argument(
        "--version",
        action="version",
        version="%(prog)s {version}".format(version=__version__),
    )
    subparsers = parser.add_subparsers(title="subcommands", dest="subcommand")

    # 'bootstrap' subcommand
    bootstrap_parser = subparsers.add_parser(
        "bootstrap", help="Download and install session-manager-plugin"
    )
    bootstrap_parser.add_argument(
        "-f", "--force", action="store_true", help="Forces bootstrap operation"
    )

    # 'session' subcommand
    session_parser = subparsers.add_parser(
        "session", help="Open new session on instance and connect to it"
    )
    session_parser.add_argument("-p", "--profile", help="AWS profile to use")
    session_parser.add_argument("-r", "--region", help="AWS region to use")
    session_parser.add_argument(
        "instance_name", help="Instance we wish to open session to"
    )

    # 'ssh' subcommand
    ssh_parser = subparsers.add_parser(
        "ssh", help="Open SSH session on instance and connect to it"
    )
    ssh_parser.add_argument("-p", "--profile", help="AWS profile to use")
    ssh_parser.add_argument("-r", "--region", help="AWS region to use")
    ssh_parser.add_argument(
        "-l", "--os-user", help="SSH user to use", type=str, default=DEFAULT_OS_USER
    )
    ssh_parser.add_argument(
        "-P", "--port", help="SSH port to use", type=int, default=DEFAULT_SSH_PORT
    )
    ssh_parser.add_argument(
        "--key-type",
        type=str,
        default=DEFAULT_KEY_ALGORITHM,
        choices=SUPPORTED_KEY_TYPES,
        help=argparse.SUPPRESS,
    )
    ssh_parser.add_argument(
        "--key-size", type=int, default=DEFAULT_KEY_SIZE, help=argparse.SUPPRESS
    )
    ssh_parser.add_argument("instance_name", help="Instance we wish to open session to")
    ssh_parser.add_argument(
        "command", help="command to execute on the instance", nargs=argparse.REMAINDER
    )

    # 'ssh_config' subcommand
    ssh_config_parser = subparsers.add_parser(
        "ssh-config", help="Generate SSH configuration file"
    )
    ssh_config_parser.add_argument("-p", "--profile", help="AWS profile to use")
    ssh_config_parser.add_argument("-r", "--region", help="AWS region to use")
    ssh_config_parser.add_argument(
        "-l", "--os-user", help="SSH user to use", type=str, default=DEFAULT_OS_USER
    )
    ssh_config_parser.add_argument(
        "-P", "--port", help="SSH port to use", type=int, default=DEFAULT_SSH_PORT
    )

    # 'ssh-proxy' subcommand
    ssh_proxy_parser = subparsers.add_parser(
        "ssh-proxy", help="Open new SSH proxy session to instance"
    )
    ssh_proxy_parser.add_argument("-p", "--profile", help="AWS profile to use")
    ssh_proxy_parser.add_argument("-r", "--region", help="AWS region to use")
    ssh_proxy_parser.add_argument(
        "-l", "--os-user", help="SSH user to use", type=str, default=DEFAULT_OS_USER
    )
    ssh_proxy_parser.add_argument(
        "-P", "--port", help="SSH port to use", type=int, default=DEFAULT_SSH_PORT
    )
    ssh_proxy_parser.add_argument(
        "--key-type",
        type=str,
        default=DEFAULT_KEY_ALGORITHM,
        choices=SUPPORTED_KEY_TYPES,
        help=argparse.SUPPRESS,
    )
    ssh_proxy_parser.add_argument(
        "--key-size", type=int, default=DEFAULT_KEY_SIZE, help=argparse.SUPPRESS
    )
    ssh_proxy_parser.add_argument(
        "instance_name", help="Instance we wish to open session to"
    )

    # 'list' subcommand
    ls_parser = subparsers.add_parser(
        "list", aliases=["ls"], help="List available instances"
    )
    ls_parser.add_argument("-p", "--profile", help="AWS profile to use")
    ls_parser.add_argument("-r", "--region", help="AWS region to use")
    ls_parser.add_argument(
        "-f",
        "--format",
        help="Output format",
        default=DEFAULT_LIST_OUTPUT,
        choices=DEFAULT_LIST_OUTPUT_FORMATS,
    )
    ls_parser.add_argument(
        "-o",
        "--output",
        help=argparse.SUPPRESS,
        default=",".join(DEFAULT_LIST_HUMAN_FIELDS),
    )

    args = parser.parse_args()

    if not args.subcommand:
        parser.print_help()
        sys.exit(1)

    return args


def main():
    args = parse_arguments()

    if not DEBUG:
        sys.excepthook = lambda exc_type, exc_value, traceback: logger.error(exc_value)

    log_level = logging.ERROR
    log_format = "%(message)s"

    # We want to silence dependencies
    logging.getLogger("botocore").setLevel(logging.CRITICAL)
    logging.getLogger("boto3").setLevel(logging.CRITICAL)
    logging.getLogger("urllib3").setLevel(logging.CRITICAL)

    if args.verbose:
        log_level = logging.INFO

    if DEBUG:
        log_level = logging.DEBUG
        log_format = "%(asctime)s - %(name)-28s - %(levelname)-5s - %(message)s"

    logging.basicConfig(level=log_level, stream=sys.stderr, format=log_format)

    try:
        config = load_config_from_files()
    except (ValidationError, ScannerError) as e:
        raise ValueError("Invalid configuration provided: {}".format(e))

    # We want to provide default values in cases they are not configured
    # in ~/.aws/config or availabe a environment variables
    default_region = get_default_region()
    if default_region is None:
        default_region = AWS_DEFAULT_REGION

    # We try to obtain default profile from the environment or use 'default' to
    # save a call to boto3. In the environment, we check if we are being called
    # from aws-vault first or not. Then we return 'default' as boto3 will
    # https://github.com/boto/boto3/blob/develop/boto3/session.py#L93
    if "AWS_VAULT" in os.environ:
        logger.debug(
            "aws-vault usage detected, defaulting to the AWS profile from $AWS_VAULT"
        )

    default_profile = (
        os.environ.get("AWS_VAULT")
        or os.environ.get("AWS_PROFILE")
        or AWS_DEFAULT_PROFILE
    )

    profile = _get_profile(args=args, config=config, default=default_profile)
    region = _get_region(args=args, config=config, default=default_region)

    logger.debug('Using AWS profile "%s" in region "%s"', profile, region)

    if args.subcommand == "bootstrap":
        bootstrap(force=args.force)
    if args.subcommand == "session":
        session(
            config=config,
            instance_name=args.instance_name,
            region_name=region,
            profile_name=profile,
        )
    if args.subcommand == "ssh":
        ssh(
            config=config,
            instance_name=args.instance_name,
            region_name=region,
            profile_name=profile,
            user=args.os_user,
            port=args.port,
            key_type=args.key_type,
            key_size=args.key_size,
            command=args.command,
        )
    if args.subcommand == "ssh-config":
        ssh_config(
            region_name=region, profile_name=profile, user=args.os_user, port=args.port
        )
    if args.subcommand == "ssh-proxy":
        ssh_proxy(
            config=config,
            instance_name=args.instance_name,
            region_name=region,
            profile_name=profile,
            user=args.os_user,
            port=args.port,
            key_type=args.key_type,
            key_size=args.key_size,
        )
    if args.subcommand in ["ls", "list"]:
        fields = args.output.split(",")
        list_instances(
            region_name=region,
            profile_name=profile,
            output_format=args.format,
            fields=fields,
        )


if __name__ == "__main__":
    main()



import ipaddress
import logging

import botocore.exceptions

from aws_gate.exceptions import AWSConnectionError

logger = logging.getLogger(__name__)


def _is_valid_ip(ip):
    try:
        ipaddress.ip_address(ip)
    except ValueError:
        return False
    return True


def _query_aws_api(filters, ec2=None):
    ret = None

    # We are always interested only in running EC2 instances as we cannot
    # open a session to terminated EC2 instance.
    filters = filters + [{"Name": "instance-state-name", "Values": ["running"]}]

    try:
        ec2_instances = list(ec2.instances.filter(Filters=filters))
        logger.debug("Found %s maching instances", len(ec2_instances))
        for i in ec2_instances:
            if i.instance_id:
                logger.debug("Matching instance: %s", i.instance_id)
                ret = i.instance_id
    except botocore.exceptions.ClientError:
        raise AWSConnectionError

    return ret


def getinstanceidbyprivatednsname(name, ec2=None):
    filters = [{"Name": "private-dns-name", "Values": [name]}]
    return _query_aws_api(filters=filters, ec2=ec2)


def getinstanceidbydnsname(name, ec2=None):
    filters = [{"Name": "dns-name", "Values": [name]}]
    return _query_aws_api(filters=filters, ec2=ec2)


def getinstanceidbyprivateipaddress(name, ec2=None):
    filters = [{"Name": "private-ip-address", "Values": [name]}]
    return _query_aws_api(filters=filters, ec2=ec2)


def getinstanceidbyipaddress(name, ec2=None):
    filters = [{"Name": "ip-address", "Values": [name]}]
    return _query_aws_api(filters=filters, ec2=ec2)


def getinstanceidbytag(name, ec2=None):
    key, value = name.split(":")

    filters = [{"Name": "tag:{}".format(key), "Values": [value]}]

    return _query_aws_api(filters=filters, ec2=ec2)


def getinstanceidbyinstancename(name, ec2=None):
    return getinstanceidbytag("Name:{}".format(name), ec2=ec2)


def query_instance(name, ec2=None):
    if ec2 is None:
        raise ValueError("EC2 client is not initialized")

    logger.debug("Querying EC2 API for instance identifier: %s", name)

    identifier_type = None
    func_dispatcher = {
        "dns-name": getinstanceidbydnsname,
        "private-dns-name": getinstanceidbyprivatednsname,
        "ip-address": getinstanceidbyipaddress,
        "private-ip-address": getinstanceidbyprivateipaddress,
        "tag": getinstanceidbytag,
        "name": getinstanceidbyinstancename,
    }

    # If we are provided with instance ID directly, we don't need to contact EC2
    # API and can return the value directly.
    if name.startswith("id-") or name.startswith("i-"):
        return name

    if _is_valid_ip(name):
        if not ipaddress.ip_address(name).is_private:
            identifier_type = "ip-address"
        else:
            identifier_type = "private-ip-address"
    else:
        if name.endswith("compute.amazonaws.com"):
            identifier_type = "dns-name"
        elif name.endswith("compute.internal"):
            identifier_type = "private-dns-name"
        elif name.count(":") == 1:
            identifier_type = "tag"
        else:
            identifier_type = "name"

    return func_dispatcher[identifier_type](name=name, ec2=ec2)



from datetime import timedelta

import pytest
from hypothesis import given, example, settings
from hypothesis.strategies import text, integers, sampled_from

from aws_gate.constants import DEFAULT_GATE_KEY_PATH
from aws_gate.ssh_common import (
    SshKey,
    SUPPORTED_KEY_TYPES,
    KEY_MIN_SIZE,
    SshKeyUploader,
)


@settings(deadline=timedelta(milliseconds=400))
@given(sampled_from(SUPPORTED_KEY_TYPES), integers(min_value=KEY_MIN_SIZE))
def test_initialize_key(key_type, key_size):
    key = SshKey(key_type=key_type, key_size=key_size)

    assert key.key_path == DEFAULT_GATE_KEY_PATH
    assert key.key_type == key_type
    assert key.key_size == key_size


@settings(deadline=timedelta(milliseconds=400))
@given(sampled_from(SUPPORTED_KEY_TYPES))
def test_initialize_key_as_context_manager(mocker, key_type):
    mocker.patch("aws_gate.ssh_common.os")
    open_mock = mocker.patch("builtins.open", new_callable=mocker.mock_open())

    with SshKey(key_type=key_type):
        assert open_mock.called
        open_mock.assert_called_with(DEFAULT_GATE_KEY_PATH, "wb")


@settings(deadline=timedelta(milliseconds=400))
@given(sampled_from(SUPPORTED_KEY_TYPES))
def test_ssh_public_key(key_type):
    key = SshKey(key_type=key_type)
    key.generate()

    if key_type == "rsa":
        key_start_str = "ssh-rsa"
    else:
        key_start_str = "ssh-ed25519"

    assert key.public_key.decode().startswith(key_start_str)


@given(integers(max_value=KEY_MIN_SIZE))
@example(0)
@example(-1024)
def test_initialize_key_unsupported_key_size(key_size):
    with pytest.raises(ValueError):
        SshKey(key_size=key_size)


def test_initialize_key_invalid_key_path():
    with pytest.raises(ValueError):
        SshKey(key_path="")


@given(text())
def test_initialize_key_unsupported_key_type(key_type):
    with pytest.raises(ValueError):
        SshKey(key_type=key_type)


def test_ssh_key_file_permissions(mocker):
    mocker.patch("builtins.open", new_callable=mocker.mock_open())
    m = mocker.patch("aws_gate.ssh_common.os.chmod")

    key = SshKey()
    key.generate()
    key.write_to_file()

    assert m.called
    assert mocker.call(DEFAULT_GATE_KEY_PATH, 0o600) == m.call_args_list[0]


def test_delete_key(mocker):
    mocker.patch("builtins.open", new_callable=mocker.mock_open())
    m = mocker.patch("aws_gate.ssh_common.os", new_callable=mocker.MagicMock())

    key = SshKey()
    key.generate()
    key.write_to_file()
    key.delete()

    assert m.remove.called
    assert m.remove.call_args == mocker.call(DEFAULT_GATE_KEY_PATH)


def test_uploader(ec2_ic_mock, ssh_key, instance_id):
    uploader = SshKeyUploader(
        instance_id=instance_id, az="eu-west-1a", ssh_key=ssh_key, ec2_ic=ec2_ic_mock
    )
    uploader.upload()

    assert ec2_ic_mock.send_ssh_public_key.called


def test_uploader_as_context_manager(ec2_ic_mock, ssh_key, instance_id):
    with SshKeyUploader(
        instance_id=instance_id, az="eu-west-1a", ssh_key=ssh_key, ec2_ic=ec2_ic_mock
    ):
        assert ec2_ic_mock.send_ssh_public_key.called


def test_uploader_exception(ec2_ic_mock, ssh_key, instance_id):
    ec2_ic_mock.configure_mock(
        **{"send_ssh_public_key.return_value": {"Success": False, "RequestId": "12345"}}
    )
    with pytest.raises(ValueError):
        uploader = SshKeyUploader(
            instance_id=instance_id,
            az="eu-west-1a",
            ssh_key=ssh_key,
            ec2_ic=ec2_ic_mock,
        )
        uploader.upload()



import pytest
from botocore.exceptions import ClientError

from aws_gate.query import _query_aws_api, query_instance, AWSConnectionError


def test_query_aws_api_exception(mocker):
    ec2_mock = mocker.MagicMock()

    # https://github.com/surbas/pg2kinesis/blob/master/tests/test_stream.py#L20
    error_response = {"Error": {"Code": "ResourceInUseException"}}
    ec2_mock.configure_mock(
        **{"instances.filter.side_effect": ClientError(error_response, "random_ec2_op")}
    )

    filters = [{"Name": "ip-address", "Values": ["10.1.1.1"]}]
    with pytest.raises(AWSConnectionError):
        _query_aws_api(filters=filters, ec2=ec2_mock)


@pytest.mark.parametrize(
    "name",
    [
        "10.69.104.49",
        "ec2-18-202-215-108.eu-west-1.compute.amazonaws.com",
        "ip-10-69-104-49.eu-west-1.compute.internal",
        "18.202.215.108",
        "i-0c32153096cd68a6d",
        "Name:dummy-instance",
        "dummy-instance",
    ],
    ids=[
        "private_ip_address",
        "dns_name",
        "private_dns_name",
        "ip_address",
        "id",
        "tag",
        "name",
    ],
)
def test_query_instance(name, instance_id, ec2):
    assert query_instance(name, ec2=ec2) == instance_id


def test_query_instance_ec2_unitialized():
    with pytest.raises(ValueError):
        query_instance("18.205.215.108")



import logging

from aws_gate.constants import (
    AWS_DEFAULT_PROFILE,
    AWS_DEFAULT_REGION,
    DEFAULT_OS_USER,
    DEFAULT_SSH_PORT,
    DEFAULT_KEY_ALGORITHM,
    DEFAULT_KEY_SIZE,
)
from aws_gate.decorators import (
    plugin_version,
    plugin_required,
    valid_aws_profile,
    valid_aws_region,
)
from aws_gate.query import query_instance
from aws_gate.session_common import BaseSession
from aws_gate.ssh_common import SshKey, SshKeyUploader
from aws_gate.utils import (
    get_aws_client,
    get_aws_resource,
    fetch_instance_details_from_config,
    get_instance_details,
)

logger = logging.getLogger(__name__)


class SshProxySession(BaseSession):
    def __init__(
        self,
        instance_id,
        ssm=None,
        region_name=AWS_DEFAULT_REGION,
        profile_name=AWS_DEFAULT_PROFILE,
        port=DEFAULT_SSH_PORT,
        user=DEFAULT_OS_USER,
    ):
        self._instance_id = instance_id
        self._region_name = region_name
        self._profile_name = profile_name if profile_name is not None else ""
        self._ssm = ssm
        self._port = port
        self._user = user

        self._session_parameters = {
            "Target": self._instance_id,
            "DocumentName": "AWS-StartSSHSession",
            "Parameters": {"portNumber": [str(self._port)]},
        }


@plugin_required
@plugin_version("1.1.23.0")
@valid_aws_profile
@valid_aws_region
def ssh_proxy(
    config,
    instance_name,
    user=DEFAULT_OS_USER,
    port=DEFAULT_SSH_PORT,
    key_type=DEFAULT_KEY_ALGORITHM,
    key_size=DEFAULT_KEY_SIZE,
    profile_name=AWS_DEFAULT_PROFILE,
    region_name=AWS_DEFAULT_REGION,
):
    instance, profile, region = fetch_instance_details_from_config(
        config, instance_name, profile_name, region_name
    )

    ssm = get_aws_client("ssm", region_name=region, profile_name=profile)
    ec2 = get_aws_resource("ec2", region_name=region, profile_name=profile)
    ec2_ic = get_aws_client(
        "ec2-instance-connect", region_name=region, profile_name=profile
    )

    instance_id = query_instance(name=instance, ec2=ec2)
    if instance_id is None:
        raise ValueError("No instance could be found for name: {}".format(instance))

    az = get_instance_details(instance_id=instance_id, ec2=ec2)["availability_zone"]

    logger.info(
        "Opening SSH proxy session on instance %s (%s) via profile %s",
        instance_id,
        region,
        profile,
    )
    with SshKey(key_type=key_type, key_size=key_size) as ssh_key:
        with SshKeyUploader(
            instance_id=instance_id, az=az, user=user, ssh_key=ssh_key, ec2_ic=ec2_ic
        ):
            with SshProxySession(
                instance_id,
                region_name=region,
                profile_name=profile,
                ssm=ssm,
                port=port,
                user=user,
            ) as ssh_proxy_session:
                ssh_proxy_session.open()



import pytest
import requests

from aws_gate.bootstrap import (
    Plugin,
    MacPlugin,
    bootstrap,
    _check_plugin_version,
    LinuxPlugin,
)
from aws_gate.constants import DEFAULT_GATE_BIN_PATH, PLUGIN_INSTALL_PATH
from aws_gate.exceptions import UnsupportedPlatormError


@pytest.mark.parametrize(
    "platform", [("Linux", "LinuxPlugin"), ("Darwin", "MacPlugin")], ids=lambda x: x[0]
)
def test_bootstrap(mocker, platform):
    mocker.patch("aws_gate.bootstrap.platform.system", return_value=platform[0])
    m = mocker.patch("aws_gate.bootstrap.{}".format(platform[1]))

    bootstrap()

    assert m.called


def test_bootstrap_unsupported_platform(mocker):
    mocker.patch("aws_gate.bootstrap.platform.system", return_value="non-existing-os")

    with pytest.raises(UnsupportedPlatormError):
        bootstrap()


def test_check_plugin_version(mocker):
    m = mocker.patch("aws_gate.bootstrap.execute")

    _check_plugin_version()

    assert m.called
    assert m.call_args == mocker.call(
        PLUGIN_INSTALL_PATH, ["--version"], capture_output=True
    )


def test_plugin_is_installed(mocker):
    mocker.patch("aws_gate.bootstrap.shutil.which", return_value=PLUGIN_INSTALL_PATH)

    plugin = Plugin()

    assert not plugin.is_installed


def test_plugin_extract_raises_notimplementederror():
    plugin = Plugin()

    with pytest.raises(NotImplementedError):
        plugin.extract()


def test_plugin_download(mocker):
    mocker.patch("aws_gate.bootstrap.os")
    mocker.patch("aws_gate.bootstrap.shutil")
    requests_mock = mocker.patch("aws_gate.bootstrap.requests")

    plugin = Plugin()
    plugin.download()

    assert requests_mock.get.called


def test_plugin_download_exception(mocker):
    mocker.patch("aws_gate.bootstrap.os")
    mocker.patch("aws_gate.bootstrap.shutil")
    mocker.patch(
        "aws_gate.bootstrap.requests.get", side_effect=requests.exceptions.HTTPError
    )
    m = mocker.patch("aws_gate.bootstrap.logger.error")

    plugin = Plugin()
    plugin.download()

    assert m.called


def test_mac_plugin_extract_invalid_zip(mocker):
    mocker.patch("aws_gate.bootstrap.zipfile.is_zipfile", return_value=False)

    plugin = MacPlugin()

    with pytest.raises(ValueError):
        plugin.extract()


def test_mac_plugin_extract_valid(mocker):
    mocker.patch("aws_gate.bootstrap.zipfile.is_zipfile", return_value=True)
    mocker.patch("aws_gate.bootstrap.os.path.split")
    zip_mock = mocker.patch("aws_gate.bootstrap.zipfile.ZipFile")

    plugin = MacPlugin()
    plugin.extract()

    assert zip_mock.called


def test_mac_plugin_install_non_existent_bin_dir(mocker):
    mocker.patch("builtins.open", mocker.mock_open(read_data="data"))
    os_mock = mocker.patch("aws_gate.bootstrap.os")
    mocker.patch("aws_gate.bootstrap.os.path.exists", return_value=False)
    mocker.patch("aws_gate.bootstrap._check_plugin_version", return_value="1.1.23.0")
    shutil_mock = mocker.patch("aws_gate.bootstrap.shutil")

    plugin = MacPlugin()
    plugin.install()

    assert os_mock.makedirs.called
    assert os_mock.makedirs.call_args == mocker.call(DEFAULT_GATE_BIN_PATH)
    assert shutil_mock.copyfileobj.called
    assert os_mock.chmod.called


def test_linux_plugin_extract_valid(mocker):
    mocker.patch("aws_gate.bootstrap.os.path.split")
    tarfile_mock = mocker.patch("aws_gate.bootstrap.tarfile")
    unix_ar_mock = mocker.patch("aws_gate.bootstrap.unix_ar")

    plugin = LinuxPlugin()
    plugin.extract()

    assert unix_ar_mock.open.called
    assert tarfile_mock.open.called



import logging
import os

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import rsa, ed25519

from aws_gate.constants import (
    DEFAULT_GATE_KEY_PATH,
    DEFAULT_KEY_SIZE,
    SUPPORTED_KEY_TYPES,
    DEFAULT_OS_USER,
)

logger = logging.getLogger(__name__)

KEY_MIN_SIZE = DEFAULT_KEY_SIZE


class SshKey:
    def __init__(
        self, key_path=DEFAULT_GATE_KEY_PATH, key_type="rsa", key_size=KEY_MIN_SIZE
    ):
        self._key_path = None
        self._key_type = None
        self._key_size = None
        self._private_key = None
        self._public_key = None

        self.key_path = key_path
        self.key_type = key_type
        self.key_size = key_size

    def __enter__(self):
        self.generate()
        self.write_to_file()
        return self

    def __exit__(self, *args):
        self.delete()

    def _generate_key(self):
        self._private_key = None

        if self._key_type == "rsa":
            self._private_key = rsa.generate_private_key(
                public_exponent=65537,
                key_size=self._key_size,
                backend=default_backend(),
            )
        elif self._key_type == "ed25519":
            self._private_key = ed25519.Ed25519PrivateKey.generate()

        self._public_key = self._private_key.public_key()

    def generate(self):
        self._generate_key()

    def write_to_file(self):
        with open(self._key_path, "wb") as f:
            f.write(self.private_key)
        # 'ssh' refuses to use the key with broad access permissions
        os.chmod(self._key_path, 0o600)

    def delete(self):
        os.remove(self._key_path)

    @property
    def public_key(self):
        return self._public_key.public_bytes(
            encoding=serialization.Encoding.OpenSSH,
            format=serialization.PublicFormat.OpenSSH,
        )

    @property
    def private_key(self):
        return self._private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption(),
        )

    @property
    def key_type(self):
        return self._key_type

    @key_type.setter
    def key_type(self, value):
        if not value or value not in SUPPORTED_KEY_TYPES:
            raise ValueError("Unsupported or invalid key type: {}".format(value))

        self._key_type = value

    @property
    def key_path(self):
        return self._key_path

    @key_path.setter
    def key_path(self, value):
        if not value:
            raise ValueError("Invalid key path: {}".format(value))
        self._key_path = value

    @property
    def key_size(self):
        return self._key_size

    @key_size.setter
    def key_size(self, value):
        if value < KEY_MIN_SIZE:
            raise ValueError("Invalid key size: {}".format(value))

        self._key_size = value


class SshKeyUploader:
    def __init__(
        self, instance_id, az, user=DEFAULT_OS_USER, ssh_key=None, ec2_ic=None
    ):
        self._instance_id = instance_id
        self._az = az
        self._ssh_key = ssh_key
        self._ec2_ic = ec2_ic
        self._user = user

    def __enter__(self):
        self.upload()
        return self

    def __exit__(self, *args):
        pass

    def upload(self):
        logger.debug("Uploading SSH public key: %s", self._ssh_key.public_key.decode())
        response = self._ec2_ic.send_ssh_public_key(
            InstanceId=self._instance_id,
            InstanceOSUser=self._user,
            SSHPublicKey=str(self._ssh_key.public_key.decode()),
            AvailabilityZone=self._az,
        )
        logger.debug("Received response: %s", response)
        if not response["Success"]:
            raise ValueError(
                "Failed to upload SSH key to instance {}".format(self._instance_id)
            )



import json
import pytest

from aws_gate.list import list_instances, serialize


def test_list(mocker, ec2, ssm, capsys):
    mocker.patch("aws_gate.list.get_aws_resource", return_value=ec2)
    mocker.patch("aws_gate.list.get_aws_client", return_value=ssm)
    mocker.patch("aws_gate.decorators.is_existing_region", return_value=True)
    mocker.patch("aws_gate.decorators.is_existing_profile", return_value=True)

    expected_data = [
        "i-0c32153096cd68a6d",
        "dummy-instance",
        "eu-west-1a",
        "vpc-1981f29759da4a354",
        "10.69.104.49\n",
    ]
    expected = " ".join(expected_data)

    list_instances(profile_name="default", region_name="eu-west-1")
    out, _ = capsys.readouterr()

    assert out == expected


def test_list_invalid_profile():
    with pytest.raises(ValueError):
        list_instances(profile_name="invalid-profile", region_name="eu-west-1")


def test_list_invalid_region(mocker):
    mocker.patch("aws_gate.decorators.is_existing_profile", return_value=True)

    with pytest.raises(ValueError):
        list_instances(profile_name="default", region_name="invalid-region")


def test_list_invalid_field(mocker):
    mocker.patch("aws_gate.decorators.is_existing_region", return_value=True)
    mocker.patch("aws_gate.decorators.is_existing_profile", return_value=True)

    with pytest.raises(ValueError):
        list_instances(
            profile_name="default", region_name="eu-west-1", fields=["invalid-field"]
        )


@pytest.mark.parametrize(
    "output_format, fields, data, expected",
    [
        (
            "json",
            ["foo"],
            [{"foo": "bar"}],
            json.dumps([{"foo": "bar"}], sort_keys=True, indent=4),
        ),
        (
            "json",
            ["foo", "bar"],
            [{"foo": "bar", "bar": "foo"}],
            json.dumps([{"foo": "bar", "bar": "foo"}], sort_keys=True, indent=4),
        ),
        ("human", ["foo"], [{"foo": "bar"}], "bar\r\n"),
        ("human", ["foo", "bar"], [{"foo": "bar", "bar": "foo"}], "bar foo\r\n"),
        ("tsv", ["foo"], [{"foo": "bar"}], "bar\r\n"),
        ("tsv", ["foo", "bar"], [{"foo": "bar", "bar": "foo"}], "bar\tfoo\r\n"),
        ("csv", ["foo"], [{"foo": "bar"}], "bar\r\n"),
        ("csv", ["foo", "bar"], [{"foo": "bar", "bar": "foo"}], "bar,foo\r\n"),
    ],
    ids=[
        "{}-{}".format(type_, number)
        for type_ in ["json", "human", "tsv", "csv"]
        for number in ["single", "many"]
    ],
)
def test_serializer(output_format, fields, data, expected):
    assert serialize(data=data, output_format=output_format, fields=fields) == expected



import logging

from aws_gate.constants import AWS_DEFAULT_PROFILE, AWS_DEFAULT_REGION
from aws_gate.decorators import (
    plugin_version,
    plugin_required,
    valid_aws_profile,
    valid_aws_region,
)
from aws_gate.query import query_instance
from aws_gate.session_common import BaseSession
from aws_gate.utils import (
    get_aws_client,
    get_aws_resource,
    fetch_instance_details_from_config,
)

logger = logging.getLogger(__name__)


class SSMSession(BaseSession):
    def __init__(
        self,
        instance_id,
        region_name=AWS_DEFAULT_REGION,
        profile_name=AWS_DEFAULT_REGION,
        ssm=None,
    ):
        self._instance_id = instance_id
        self._region_name = region_name
        self._profile_name = profile_name if profile_name is not None else ""
        self._ssm = ssm

        self._session_parameters = {"Target": self._instance_id}


@plugin_required
@plugin_version("1.1.23.0")
@valid_aws_profile
@valid_aws_region
def session(
    config,
    instance_name,
    profile_name=AWS_DEFAULT_PROFILE,
    region_name=AWS_DEFAULT_REGION,
):
    instance, profile, region = fetch_instance_details_from_config(
        config, instance_name, profile_name, region_name
    )

    ssm = get_aws_client("ssm", region_name=region, profile_name=profile)
    ec2 = get_aws_resource("ec2", region_name=region, profile_name=profile)

    instance_id = query_instance(name=instance, ec2=ec2)
    if instance_id is None:
        raise ValueError("No instance could be found for name: {}".format(instance))

    logger.info(
        "Opening session on instance %s (%s) via profile %s",
        instance_id,
        region,
        profile,
    )
    with SSMSession(instance_id, region_name=region, ssm=ssm) as sess:
        sess.open()



import inspect
import os

from setuptools import setup, find_packages

from aws_gate import __version__, __description__, __author__, __author_email__, __url__

__location__ = os.path.join(
    os.getcwd(), os.path.dirname(inspect.getfile(inspect.currentframe()))
)


def get_install_requirements(path):
    with open(os.path.join(__location__, path), 'r') as f:
        content = f.read()
        requires = [req for req in content.split('\\n') if req != '']

    return requires


NAME = 'aws-gate'
PROJECT_URLS = {
    "Bug Tracker": "https://github.com/xen0l/aws-gate/issues",
    "Source Code": "https://github.com/xen0l/aws-gate",
}
CLASSIFIERS = [
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: BSD License",
    "Operating System :: OS Independent",
    "Topic :: System :: Systems Administration",
]
INSTALL_REQUIRES = get_install_requirements('requirements/requirements.txt')
EXTRA_REQUIRES = {
    'tests': get_install_requirements('requirements/requirements_dev.txt')
}
SCRIPTS = [
    'bin/aws-gate'
]

setup(
    name=NAME,
    version=__version__,
    description=__description__,
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    url=__url__,
    project_urls=PROJECT_URLS,
    author=__author__,
    author_email=__author_email__,
    packages=find_packages(),
    classifiers=CLASSIFIERS,
    install_requires=INSTALL_REQUIRES,
    extra_requires=EXTRA_REQUIRES,
    include_package_data=True,
    scripts=SCRIPTS
)



__version__ = "0.8.1"
__description__ = "aws-gate - AWS SSM Session Manager client CLI"
__author__ = "Adam Stevko"
__author_email__ = "adam.stevko@gmail.com"
__url__ = "https://github.com/xen0l/aws-gate"



import logging
import os
import platform
import shutil
import tarfile
import tempfile
import zipfile

import requests
import unix_ar

from aws_gate.constants import (
    DEFAULT_GATE_BIN_PATH,
    PLUGIN_INSTALL_PATH,
    PLUGIN_NAME,
    SSM_PLUGIN_PATH,
)
from aws_gate.exceptions import UnsupportedPlatormError
from aws_gate.utils import execute

logger = logging.getLogger(__name__)


def _check_plugin_version(path=PLUGIN_INSTALL_PATH):
    return execute(path, ["--version"], capture_output=True)


class Plugin:
    url = None
    download_path = None

    @property
    def is_installed(self):
        logger.debug("Checking if %s exists and is executable", PLUGIN_INSTALL_PATH)
        return shutil.which(PLUGIN_INSTALL_PATH) is None

    def download(self):
        tmp_dir = tempfile.mkdtemp()
        file_name = os.path.split(self.url)[-1]

        self.download_path = os.path.join(tmp_dir, file_name)
        try:
            logger.debug("Downloading session-manager-plugin archive from %s", self.url)
            with requests.get(self.url, stream=True) as req:
                req.raise_for_status()
                with open(self.download_path, "wb") as f:
                    shutil.copyfileobj(req.raw, f)
                    logger.debug("Download stored at %s", self.download_path)
        except requests.exceptions.HTTPError as e:
            logger.error("HTTP error while downloading %s: %s", self.url, e)

    def install(self):
        download_dir = os.path.split(self.download_path)[0]
        plugin_src_path = os.path.join(
            download_dir, SSM_PLUGIN_PATH[platform.system()]["bundle"]
        )
        plugin_dst_path = PLUGIN_INSTALL_PATH

        if not os.path.exists(DEFAULT_GATE_BIN_PATH):
            logger.debug("Creating %s", DEFAULT_GATE_BIN_PATH)
            os.makedirs(DEFAULT_GATE_BIN_PATH)

        with open(plugin_src_path, "rb") as f_src, open(plugin_dst_path, "wb") as f_dst:
            logger.debug("Copying %s to %s", plugin_src_path, plugin_dst_path)
            shutil.copyfileobj(f_src, f_dst)

        logger.debug("Setting execution permissions on %s", plugin_dst_path)
        os.chmod(plugin_dst_path, 0o755)

        version = _check_plugin_version(PLUGIN_INSTALL_PATH)
        print("{} (version {}) installed successfully!".format(PLUGIN_NAME, version))

    def extract(self):
        raise NotImplementedError


class MacPlugin(Plugin):
    url = SSM_PLUGIN_PATH["Darwin"]["download"]

    def extract(self):
        if not zipfile.is_zipfile(self.download_path):
            raise ValueError(
                "Invalid macOS session-manager-plugin ZIP file found {}".format(
                    self.download_path
                )
            )

        with zipfile.ZipFile(self.download_path, "r") as zip_file:
            download_dir = os.path.split(self.download_path)[0]
            logger.debug(
                "Extracting session-manager-plugin archive at %s", download_dir
            )
            zip_file.extractall(download_dir)


class LinuxPlugin(Plugin):
    url = SSM_PLUGIN_PATH["Linux"]["download"]

    def extract(self):
        ar_file = unix_ar.open(self.download_path)
        tarball = ar_file.open("data.tar.gz/")
        tar_file = tarfile.open(fileobj=tarball)
        download_dir = os.path.split(self.download_path)[0]
        logger.debug("Extracting session-manager-plugin archive at %s", download_dir)
        tar_file.extractall(download_dir)


def bootstrap(force=False):
    system = platform.system()
    if system == "Darwin":
        plugin = MacPlugin()
    elif system == "Linux":
        plugin = LinuxPlugin()
    else:
        raise UnsupportedPlatormError(
            "Unable to bootstrap session-manager-plugin on {}".format(system)
        )

    if plugin.is_installed or force:
        plugin.download()
        plugin.extract()
        plugin.install()



import pytest

from aws_gate.decorators import (
    plugin_required,
    plugin_version,
    _plugin_exists,
    valid_aws_profile,
    valid_aws_region,
)


def test_plugin_exists(mocker):
    m = mocker.patch("aws_gate.decorators.os.path.exists")

    _plugin_exists("foo")

    assert m.called
    assert m.call_args == mocker.call("foo")


def test_plugin_required(mocker):
    mocker.patch("aws_gate.decorators._plugin_exists", return_value=True)

    @plugin_required
    def test_function():
        return "executed"

    assert test_function() == "executed"


def test_plugin_required_plugin_not_installed(mocker):
    mocker.patch("aws_gate.decorators._plugin_exists", return_value=False)

    @plugin_required
    def test_function():
        return "executed"

    with pytest.raises(OSError):
        test_function()


def test_plugin_version(mocker):
    m = mocker.patch("aws_gate.decorators.execute_plugin", return_value="1.1.23.0")

    @plugin_version("1.1.23.0")
    def test_function():
        return "executed"

    assert test_function() == "executed"
    assert m.call_args == mocker.call(["--version"], capture_output=True)


def test_plugin_version_bad_version(mocker):
    mocker.patch("aws_gate.decorators.execute_plugin", return_value="1.1.23.0")

    @plugin_version("1.1.25.0")
    def test_function():
        return "executed"

    with pytest.raises(ValueError):
        test_function()


def test_valid_aws_profile(mocker):
    mocker.patch("aws_gate.decorators.is_existing_profile", return_value=True)

    @valid_aws_profile
    def test_function(profile_name):
        return profile_name

    assert test_function(profile_name="profile") == "profile"


def test_valid_aws_profile_invalid_profile(mocker):
    mocker.patch("aws_gate.decorators.is_existing_profile", return_value=False)

    @valid_aws_profile
    def test_function(profile_name):
        return profile_name

    with pytest.raises(ValueError):
        test_function(profile_name="invalid-profile")


def test_valid_aws_region(mocker):
    mocker.patch("aws_gate.decorators.is_existing_region", return_value=True)

    @valid_aws_region
    def test_function(region_name):
        return region_name

    assert test_function(region_name="eu-west-1") == "eu-west-1"


def test_valid_aws_region_invalid_region(mocker):
    mocker.patch("aws_gate.decorators.is_existing_region", return_value=False)

    @valid_aws_region
    def test_function(region_name):
        return region_name

    with pytest.raises(ValueError):
        test_function(region_name="invalid-region")



import pytest

from aws_gate.ssh import SshSession, ssh


def test_create_ssh_session(ssm_mock, instance_id):
    sess = SshSession(instance_id=instance_id, ssm=ssm_mock)
    sess.create()

    assert ssm_mock.start_session.called


def test_terminate_ssh_session(ssm_mock, instance_id):
    sess = SshSession(instance_id=instance_id, ssm=ssm_mock)

    sess.create()
    sess.terminate()

    assert ssm_mock.terminate_session.called


@pytest.mark.parametrize(
    "test_input", [(False, "-q"), (True, "-vv")], ids=["DEBUG=False", "DEBUG=True"]
)
def test_open_ssh_session(mocker, ssm_mock, instance_id, test_input):
    debug, ssh_cmd_arg = test_input
    mocker.patch("aws_gate.ssh.DEBUG", debug)
    m = mocker.patch("aws_gate.ssh.execute", return_value="output")

    sess = SshSession(instance_id=instance_id, ssm=ssm_mock)
    sess.open()

    assert m.called
    assert ssh_cmd_arg in m.call_args[0][1]


def test_open_ssh_session_with_command(mocker, instance_id, ssm_mock):
    m = mocker.patch("aws_gate.ssh.execute", return_value="output")

    sess = SshSession(instance_id=instance_id, ssm=ssm_mock, command=["ls", "-l"])
    sess.open()

    assert m.called
    assert "--" in m.call_args[0][1]
    assert ["ls", "-l"] == m.call_args[0][1][-2:]


def test_open_ssh_session_host_key_verification(mocker, instance_id, ssm_mock):
    m = mocker.patch("aws_gate.ssh.execute", return_value="output")

    sess = SshSession(instance_id=instance_id, ssm=ssm_mock)
    sess.open()

    assert m.called
    assert "UserKnownHostsFile=/dev/null" in m.call_args[0][1]
    assert "StrictHostKeyChecking=no" in m.call_args[0][1]


def test_open_ssh_session_context_manager(instance_id, ssm_mock):
    with SshSession(instance_id=instance_id, ssm=ssm_mock):
        pass

    assert ssm_mock.start_session.called
    assert ssm_mock.terminate_session.called


def test_ssh_session(
    mocker, instance_id, ssh_key, get_instance_details_response, config
):
    mocker.patch("aws_gate.ssh.get_aws_client")
    mocker.patch("aws_gate.ssh.get_aws_resource")
    mocker.patch("aws_gate.ssh.query_instance", return_value=instance_id)
    ssh_session_mock = mocker.patch("aws_gate.ssh.SshSession")
    mocker.patch("aws_gate.ssh.SshKey", return_value=ssh_key)
    mocker.patch("aws_gate.ssh.SshKeyUploader")
    mocker.patch(
        "aws_gate.ssh.get_instance_details", return_value=get_instance_details_response
    )
    mocker.patch("aws_gate.decorators._plugin_exists", return_value=True)
    mocker.patch("aws_gate.decorators.execute_plugin", return_value="1.1.23.0")
    mocker.patch("aws_gate.decorators.is_existing_profile", return_value=True)

    ssh(
        config=config,
        instance_name=instance_id,
        profile_name="profile",
        region_name="eu-west-1",
    )

    assert ssh_session_mock.called


def test_ssh_exception_invalid_profile(mocker, instance_id, ssh_key, config):
    mocker.patch("aws_gate.ssh.get_aws_client")
    mocker.patch("aws_gate.ssh.get_aws_resource")
    mocker.patch("aws_gate.ssh.query_instance", return_value=None)
    mocker.patch("aws_gate.ssh.SshKey", return_value=ssh_key)
    mocker.patch("aws_gate.decorators._plugin_exists", return_value=True)
    mocker.patch("aws_gate.decorators.execute_plugin", return_value="1.1.23.0")
    mocker.patch("aws_gate.decorators.is_existing_region", return_value=True)

    with pytest.raises(ValueError):
        ssh(
            config=config,
            profile_name="invalid-profile",
            instance_name=instance_id,
            region_name="eu-west-1",
        )


def test_ssh_exception_invalid_region(mocker, instance_id, ssh_key, config):
    mocker.patch("aws_gate.ssh.get_aws_client")
    mocker.patch("aws_gate.ssh.get_aws_resource")
    mocker.patch("aws_gate.ssh.query_instance", return_value=None)
    mocker.patch("aws_gate.ssh.SshKey", return_value=ssh_key)
    mocker.patch("aws_gate.decorators._plugin_exists", return_value=True)
    mocker.patch("aws_gate.decorators.execute_plugin", return_value="1.1.23.0")
    mocker.patch("aws_gate.decorators.is_existing_profile", return_value=True)

    with pytest.raises(ValueError):
        ssh(
            config=config,
            region_name="invalid-region",
            instance_name=instance_id,
            profile_name="default",
        )


def test_ssh_exception_unknown_instance_id(mocker, instance_id, ssh_key, config):
    mocker.patch("aws_gate.ssh.get_aws_client")
    mocker.patch("aws_gate.ssh.get_aws_resource")
    mocker.patch("aws_gate.ssh.query_instance", return_value=None)
    mocker.patch("aws_gate.ssh.SshKey", return_value=ssh_key)
    mocker.patch("aws_gate.decorators._plugin_exists", return_value=True)
    mocker.patch("aws_gate.decorators.execute_plugin", return_value="1.1.23.0")
    mocker.patch("aws_gate.decorators.is_existing_profile", return_value=True)
    mocker.patch("aws_gate.decorators.is_existing_region", return_value=True)

    with pytest.raises(ValueError):
        ssh(
            config=config,
            instance_name=instance_id,
            profile_name="default",
            region_name="eu-west-1",
        )


def test_ssh_without_config(mocker, instance_id, ssh_key, empty_config):
    mocker.patch("aws_gate.ssh.get_aws_client")
    mocker.patch("aws_gate.ssh.get_aws_resource")
    mocker.patch("aws_gate.ssh.query_instance", return_value=None)
    mocker.patch("aws_gate.ssh.SshKey", return_value=ssh_key)
    mocker.patch("aws_gate.decorators._plugin_exists", return_value=True)
    mocker.patch("aws_gate.decorators.execute_plugin", return_value="1.1.23.0")

    with pytest.raises(ValueError):
        ssh(
            config=empty_config,
            instance_name=instance_id,
            profile_name="default",
            region_name="eu-west-1",
        )



import os

DEBUG = "GATE_DEBUG" in os.environ

AWS_DEFAULT_REGION = "eu-west-1"
AWS_DEFAULT_PROFILE = "default"

DEFAULT_OS_USER = "ec2-user"
DEFAULT_SSH_PORT = 22
DEFAULT_KEY_ALGORITHM = "rsa"
DEFAULT_KEY_SIZE = 2048
SUPPORTED_KEY_TYPES = ["rsa", "ed25519"]
KEY_MIN_SIZE = DEFAULT_KEY_SIZE

DEFAULT_LIST_OUTPUT = "human"
DEFAULT_LIST_OUTPUT_FORMATS = ["json", "human", "tsv", "csv"]
DEFAULT_LIST_OUTPUT_FIELDS = (
    "instance_id",
    "instance_name",
    "availability_zone",
    "vpc_id",
    "private_ip_address",
    "public_ip_addess",
    "private_dns_name",
    "public_dns_name",
)
DEFAULT_LIST_HUMAN_FIELDS = (
    "instance_id",
    "instance_name",
    "availability_zone",
    "vpc_id",
    "private_ip_address",
)

DEFAULT_GATE_DIR = os.path.expanduser("~/.aws-gate")
DEFAULT_GATE_CONFIG_PATH = os.path.join(DEFAULT_GATE_DIR, "config")
DEFAULT_GATE_CONFIGD_PATH = os.path.join(DEFAULT_GATE_DIR, "config.d")

PLUGIN_NAME = "session-manager-plugin"
DEFAULT_GATE_BIN_PATH = os.path.join(DEFAULT_GATE_DIR, "bin")
PLUGIN_INSTALL_PATH = os.path.join(DEFAULT_GATE_BIN_PATH, PLUGIN_NAME)

DEFAULT_GATE_KEY_PATH = os.path.join(DEFAULT_GATE_DIR, "key")

SSM_PLUGIN_BASE_URL = "https://s3.amazonaws.com/session-manager-downloads/plugin/latest"
SSM_PLUGIN_PATH = {
    "Darwin": {
        "download": SSM_PLUGIN_BASE_URL + "/mac/sessionmanager-bundle.zip",
        "bundle": os.path.join("sessionmanager-bundle", "bin", PLUGIN_NAME),
    },
    "Linux": {
        "download": SSM_PLUGIN_BASE_URL + "/ubuntu_64bit/session-manager-plugin.deb",
        "bundle": os.path.join(
            "usr", "local", "sessionmanagerplugin", "bin", PLUGIN_NAME
        ),
    },
}



import os

import pytest
from marshmallow import ValidationError

from aws_gate.config import (
    GateConfig,
    EmptyConfigurationError,
    load_config_from_files,
    _locate_config_files,
    validate_profile,
    validate_region,
)
from aws_gate.constants import DEFAULT_GATE_CONFIG_PATH, DEFAULT_GATE_CONFIGD_PATH


def _load_config_files(config_files):
    config = load_config_from_files(config_files=config_files)
    assert isinstance(config, GateConfig)
    return config


@pytest.mark.parametrize(
    "test_input",
    [
        ("config_invalid.yaml", ValidationError),
        ("config_invalid_attribute.yaml", ValidationError),
        ("config_empty.yaml", EmptyConfigurationError),
        ("config_invalid_yaml.yaml", EmptyConfigurationError),
    ],
    ids=lambda x: x[0].split(".")[0],
)
def test_invalid_config(shared_datadir, test_input):
    test_file, exception = test_input
    with pytest.raises(exception):
        config_files = [shared_datadir / test_file]
        _load_config_files(config_files=config_files)


def test_valid_config(shared_datadir, mocker):
    mocker.patch("aws_gate.config.is_existing_profile", return_value=True)
    expected_config = {
        "defaults": {"profile": "default-profile", "region": "eu-west-1"},
        "hosts": [
            {
                "alias": "foobar",
                "name": "foobar",
                "profile": "test-profile",
                "region": "eu-west-1",
            }
        ],
    }

    config = _load_config_files(config_files=[shared_datadir / "config_valid.yaml"])

    assert config.defaults == expected_config["defaults"]
    assert config.hosts == expected_config["hosts"]
    assert config.default_profile == expected_config["defaults"]["profile"]
    assert config.default_region == expected_config["defaults"]["region"]


def test_valid_config_without_hosts(shared_datadir, mocker):
    mocker.patch("aws_gate.config.is_existing_profile", return_value=True)
    expected_config = {
        "defaults": {"profile": "default-profile", "region": "eu-west-1"}
    }

    config = _load_config_files(
        config_files=[shared_datadir / "config_valid_without_hosts.yaml"]
    )

    assert config.defaults == expected_config["defaults"]
    assert config.hosts == []
    assert config.default_profile == expected_config["defaults"]["profile"]
    assert config.default_region == expected_config["defaults"]["region"]


def test_valid_config_without_defaults(shared_datadir):
    with pytest.raises(ValidationError):
        _load_config_files(
            config_files=[shared_datadir / "config_valid_without_defaults.yaml"]
        )


def test_valid_config_without_defaults_mocked(shared_datadir, mocker):
    mocker.patch("aws_gate.config.is_existing_profile", return_value=True)

    config = _load_config_files(
        config_files=[shared_datadir / "config_valid_without_defaults.yaml"]
    )

    assert config.default_profile is None
    assert config.default_region is None


def test_configd(shared_datadir, mocker):
    mocker.patch("aws_gate.config.is_existing_profile", return_value=True)

    expected_config = {
        "defaults": {"profile": "default-profile", "region": "eu-west-1"},
        "hosts": [
            {
                "alias": "bar",
                "name": "bar",
                "profile": "test-profile",
                "region": "eu-west-1",
            },
            {
                "alias": "foo",
                "name": "foo",
                "profile": "test-profile",
                "region": "eu-west-1",
            },
        ],
    }
    config_files = [
        os.path.join(shared_datadir, "configd", file)
        for file in sorted(os.listdir(shared_datadir / "configd"))
    ]
    config = _load_config_files(config_files=config_files)

    assert config.defaults == expected_config["defaults"]
    assert config.hosts == expected_config["hosts"]


def test_locate_config_files(mocker):
    mocker.patch("aws_gate.config.os.path.isdir", return_value=True)
    mocker.patch("aws_gate.config.os.listdir", return_value=["foo.yaml"])
    mocker.patch("aws_gate.config.os.path.isfile", return_value=True)
    expected_config_files = [
        os.path.join(DEFAULT_GATE_CONFIGD_PATH, "foo.yaml"),
        DEFAULT_GATE_CONFIG_PATH,
    ]
    assert _locate_config_files() == expected_config_files


def test_config_get_host(shared_datadir, mocker):
    mocker.patch("aws_gate.config.is_existing_profile", return_value=True)

    expected_host = {
        "alias": "foobar",
        "name": "foobar",
        "region": "eu-west-1",
        "profile": "test-profile",
    }

    config = _load_config_files(config_files=[shared_datadir / "config_get_host.yaml"])

    assert config.get_host("foobar") == expected_host
    assert config.get_host("non-existent") == {}


def test_validate_profile(mocker):
    mocker.patch("aws_gate.config.is_existing_profile", return_value=False)
    with pytest.raises(ValidationError):
        validate_profile("test-profile")


def test_validate_region(mocker):
    mocker.patch("aws_gate.config.is_existing_region", return_value=False)
    with pytest.raises(ValidationError):
        validate_region("test-region")
